/**
* DataModels.ts
* @author Abhilash Panwar (abpanwar)
* @copyright Microsoft 2020
* File containing public interfaces for EI.
*/
/**
 * The configuration to initialize the Analytics class with.
 */
export interface IAnalyticsConfiguration {
    /**
     * Ingestion key that should be used to send the events.
     */
    ingestionKey: string;
    /**
     * Endpoint where the requests containing the events should be sent.
     */
    endpointUrl?: string;
    /**
     * Autocapture configurations.
     */
    autoCapture?: IAutoCapture;
    /**
     * Set to true to let the SDK know that user has consented to storing data which can be used to
     * track user/session/etc. If set to false SDK will not store any data in the browser and will delete
     * previously stored cookies or localstorage keys.
     * Default is true.
     */
    userConsent?: boolean;
    /**
     * send usermapping events.
     */
    userMapping?: boolean;
}
/**
 * Configuration to specify the autocapture behavior of the SDK.
 */
export interface IAutoCapture {
    /**
     * Set to true to auto collect page views. Will not work with SPA's.
     */
    view?: boolean;
    /**
     * Set to false to auto collect clicks on the page.
     */
    click?: boolean;
}
/**
 * The set of attributes for Engagement Insights data for all observational data. This interface exposes the properties in a
 * Engagement Insights event.
 */
export interface IEvent {
    /**
     * Name of the event.
     */
    name: string;
    /**
     * Date object when event was emitted.
     */
    timestamp?: Date;
    /**
     * The time zone in ISO 8601 time zone format.
     */
    timeZone?: string;
    /**
     * Version of event as defined by the emitter.
     */
    version?: string;
    /**
     * Custom properties that are part of this event as defined by the emitter.
     */
    properties?: {
        [key: string]: string | number | boolean | Date;
    };
    /**
     * Ingestion key that should be used to send the event. If not specified, uses
     * the default value that is passed key passed during initialize.
     */
    ingestionKey?: string;
}
/**
 * The set of attributes in a Engagement Insights View event.
 */
export interface IView {
    /**
     * The name of the view.
     */
    name?: string;
    /**
     * The title for the view.
     */
    title?: string;
    /**
     * The referrer for the view.
     */
    referrer?: IReferrer;
    /**
     * Uri of the page.
     */
    uri: string;
    /**
     * ViewClass for the view.
     */
    viewClass?: string;
    /**
     * Date object when event was emitted.
     */
    timestamp?: Date;
    /**
     * Custom properties that are part of this view event as defined by the emitter.
     */
    properties?: {
        [key: string]: string | number | boolean | Date;
    };
}
/**
 * Attributes to specify the referrer for the page.
 */
export interface IReferrer {
    /**
     * Uri of the referrer.
     */
    uri?: string;
    /**
     * Page title of the referrer.
     */
    pageTitle?: string;
}
/**
 * The set of attributes in a Engagement Insights action event.
 */
export interface IAction {
    /**
     * Name of the action.
     */
    name?: string;
    /**
     * Type of action. Value should be one of the types defined by ActionType.
     */
    type: string;
    /**
     * Target of the action.
     */
    viewTarget?: string;
    /**
     * Id for the element on which the action occurred.
     */
    elementId?: string;
    /**
     * Class for the action.
     */
    class?: string;
    /**
     * Click coordinates where the action occured.
     */
    clickCoordinates?: string;
    /**
     * Date object when event was emitted.
     */
    timestamp?: Date;
    /**
     * Custom properties that are part of this view event as defined by the emitter.
     */
    properties?: {
        [key: string]: string | number | boolean | Date;
    };
}
/**
 * The set of attributes for a user in Engagement Insights.
 */
export interface IUser {
    /**
     * The local id for the user.
     */
    localId?: string;
    /**
     * Authenticated id for the user.
     */
    authId?: string;
    /**
     * Authentication type used to get the authenticated id for the user.
     */
    authType?: string;
    /**
     * Name of the user.
     */
    name?: string;
    /**
     * Email of the user.
     */
    email?: string;
}
