import { __extends } from "tslib";
import { TestClass } from './TestClass';
import * as sinon from 'sinon';
import Analytics from '../src/Analytics';
import Properties from '../src/properties/Properties';
import { PostChannel } from '@microsoft/1ds-post-js';
import Autocollector from '../src/analytics/Autocollector';
import { AppInsightsCore } from '@microsoft/1ds-core-js';
import { ActionType } from '../src/Index';
import SessionManager from '../src/properties/SessionManager';
import StorageManager from '../src/properties/StorageManager';
var AnalyticsTest = /** @class */ (function (_super) {
    __extends(AnalyticsTest, _super);
    function AnalyticsTest() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AnalyticsTest.prototype.testInitialize = function () {
        this.analytics = new Analytics();
        this.analytics.initialize({ ingestionKey: 'xyz', userMapping: true });
    };
    AnalyticsTest.prototype.testCleanup = function () {
        window.localStorage.clear();
    };
    AnalyticsTest.prototype.registerTests = function () {
        var _this = this;
        this.testCase({
            name: 'Verify setUser calls properties',
            test: function () {
                var propertiesStub = sinon.createStubInstance(Properties);
                var calledUser;
                _this.analytics._properties = propertiesStub;
                propertiesStub.setUser.callsFake(function (arg) {
                    calledUser = arg;
                });
                _this.analytics.setUser({ name: 'abc', localId: 'id' });
                QUnit.assert.equal(calledUser.name, 'abc');
                QUnit.assert.equal(calledUser.localId, 'id');
            }
        });
        this.testCase({
            name: 'Verify setUser updates user mapping correctly',
            test: function () {
                var appInsightsCoreStub = sinon.createStubInstance(AppInsightsCore);
                _this.analytics._applicationInsightsCore = appInsightsCoreStub;
                var event;
                appInsightsCoreStub.track.callsFake(function (arg) {
                    event = arg;
                });
                _this.analytics.setUser({ localId: 'localId', authId: 'authId' });
                QUnit.assert.equal(event.name, 'usermapping');
                // setting the same user does not send another usermapping event
                event = null;
                _this.analytics.setUser({ localId: 'localId', authId: 'authId' });
                QUnit.assert.notOk(event);
                // deleting the usermapping storage, then setting the user will send a new usermapping event
                var storageManager = new StorageManager(false);
                storageManager._setProperty('EIMappings', JSON.stringify({}));
                _this.analytics.setUser({ localId: 'localId', authId: 'authId' });
                QUnit.assert.equal(event.name, 'usermapping');
            }
        });
        this.testCase({
            name: 'Verify calling setUser before initialize does not send an event',
            test: function () {
                var appInsightsCoreStub = sinon.createStubInstance(AppInsightsCore);
                _this.analytics = new Analytics();
                _this.analytics._applicationInsightsCore = appInsightsCoreStub;
                var event;
                appInsightsCoreStub.track.callsFake(function (arg) {
                    event = arg;
                });
                _this.analytics._user = null;
                _this.analytics.setUser({ localId: 'localId', authId: 'authId' });
                QUnit.assert.notOk(event);
                QUnit.assert.equal(_this.analytics._setUserBeforeInitialize, true);
                QUnit.assert.equal(_this.analytics._user.localId, 'localId');
                QUnit.assert.equal(_this.analytics._user.authId, 'authId');
            }
        });
        this.testCase({
            name: 'Verify setProperty calls properties',
            test: function () {
                var propertiesStub = sinon.createStubInstance(Properties);
                var calledKvp;
                _this.analytics._properties = propertiesStub;
                propertiesStub.setProperty.callsFake(function (arg1, arg2) {
                    calledKvp = arg1 + arg2;
                });
                _this.analytics.setProperty('key', 'value');
                QUnit.assert.equal(calledKvp, 'keyvalue');
            }
        });
        this.testCase({
            name: 'Verify teardown calls postchannel and autocollector',
            test: function () {
                var postChannelStub = sinon.createStubInstance(PostChannel);
                var autocollectorStub = sinon.createStubInstance(Autocollector);
                _this.analytics._postChannel = postChannelStub;
                _this.analytics._autoCollector = autocollectorStub;
                _this.analytics.teardown();
                QUnit.assert.ok(autocollectorStub._removeAllListeners.calledOnce);
                QUnit.assert.ok(postChannelStub.teardown.calledOnce);
            }
        });
        this.testCase({
            name: 'Verify trackEvent',
            test: function () {
                var appInsightsCoreStub = sinon.createStubInstance(AppInsightsCore);
                _this.analytics._applicationInsightsCore = appInsightsCoreStub;
                var event = {
                    name: 'custom',
                    properties: {
                        'prop1': 'val',
                        'prop2': 10,
                        'prop3': false
                    },
                    version: '1',
                    ingestionKey: 'abc'
                };
                var item;
                appInsightsCoreStub.track.callsFake(function (arg) {
                    item = arg;
                });
                _this.analytics.trackEvent(event);
                QUnit.assert.equal(item.name, 'custom');
                QUnit.assert.equal(item.iKey, 'abc');
                QUnit.assert.ok(item.time);
                QUnit.assert.equal(item.data['prop1'], 'val');
                QUnit.assert.equal(item.data['prop2'], 10);
                QUnit.assert.equal(item.data['prop3'], false);
                QUnit.assert.equal(item.data['s']['e']['v'], '1');
            }
        });
        this.testCase({
            name: 'Verify that custom properties can not start with event. or s. or be less than 3 chars',
            test: function () {
                var appInsightsCoreStub = sinon.createStubInstance(AppInsightsCore);
                _this.analytics._applicationInsightsCore = appInsightsCoreStub;
                var event = {
                    name: 'custom',
                    properties: {
                        'Signal.test': 'val',
                        's.test': 10,
                        'ab': 20,
                        'Signal': 'value'
                    }
                };
                var item;
                appInsightsCoreStub.track.callsFake(function (arg) {
                    item = arg;
                });
                _this.analytics.setProperty('Signal.blah', 'blah');
                _this.analytics.trackEvent(event);
                QUnit.assert.notOk(item.data['Signal.test']);
                QUnit.assert.notOk(item.data['Signal.blah']);
                QUnit.assert.notOk(item.data['s.test']);
                QUnit.assert.notOk(item.data['ab']);
                QUnit.assert.notOk(item.data['Signal']);
            }
        });
        this.testCase({
            name: 'Verify trackView',
            test: function () {
                var appInsightsCoreStub = sinon.createStubInstance(AppInsightsCore);
                _this.analytics._applicationInsightsCore = appInsightsCoreStub;
                var date = new Date();
                var view = {
                    name: 'page1',
                    title: 'title',
                    uri: 'uri',
                    referrer: {
                        pageTitle: 'refPage',
                        uri: 'refUri'
                    },
                    viewClass: 'class',
                    timestamp: date,
                    properties: {
                        'prop': 'val'
                    }
                };
                var event;
                appInsightsCoreStub.track.callsFake(function (arg) {
                    event = arg;
                });
                _this.analytics.trackView(view);
                var screenInfo = _this.analytics._autoCollector._getScreenResolution();
                QUnit.assert.equal(event.name, 'view');
                QUnit.assert.equal(event.iKey, 'xyz');
                QUnit.assert.equal(event.time, date.toISOString());
                QUnit.assert.equal(event.data['s']['v']['n'], 'page1');
                QUnit.assert.equal(event.data['s']['v']['v'], 'title');
                QUnit.assert.equal(event.data['s']['v']['u'], 'uri');
                QUnit.assert.equal(event.data['s']['v']['c'], 'class');
                QUnit.assert.equal(event.data['s']['v']['h'], screenInfo.h);
                QUnit.assert.equal(event.data['s']['v']['w'], screenInfo.w);
                QUnit.assert.equal(event.data['s']['v']['g'], screenInfo.vH);
                QUnit.assert.equal(event.data['s']['v']['d'], screenInfo.vW);
                QUnit.assert.equal(event.data['s']['v']['o'], screenInfo.cd);
                QUnit.assert.equal(event.data['s']['v']['s'], screenInfo.sR);
                QUnit.assert.equal(event.data['s']['r']['u'], 'refUri');
                QUnit.assert.equal(event.data['s']['r']['p'], 'refPage');
                QUnit.assert.equal(event.data['prop'], 'val');
                // No previous view if there is no ongoing session. First view starts the session.
                QUnit.assert.notOk(event.data['s']['v']['r']);
                _this.analytics.trackView(view);
                // Second view should log the earlier view events uri as the PreviousView. 
                QUnit.assert.equal(event.data['s']['v']['r'], 'uri');
            }
        });
        this.testCase({
            name: 'Verify trackAction',
            test: function () {
                var appInsightsCoreStub = sinon.createStubInstance(AppInsightsCore);
                _this.analytics._applicationInsightsCore = appInsightsCoreStub;
                var date1 = new Date();
                var date2 = new Date(date1.getTime() + 10);
                var view = {
                    name: 'page1',
                    title: 'title',
                    uri: 'uri',
                    referrer: {
                        pageTitle: 'refPage',
                        uri: 'refUri'
                    },
                    viewClass: 'class',
                    timestamp: date1,
                    properties: {
                        'prop': 'val'
                    }
                };
                var action = {
                    name: 'cart',
                    type: ActionType.leftClick,
                    elementId: 'elId',
                    "class": 'class',
                    clickCoordinates: '20X12',
                    viewTarget: 'uri2',
                    timestamp: date2,
                    properties: {
                        'prop': 'val'
                    }
                };
                var event;
                appInsightsCoreStub.track.callsFake(function (arg) {
                    event = arg;
                });
                _this.analytics.trackView(view);
                var sessionManagerStub = sinon.createStubInstance(SessionManager);
                _this.analytics._sessionManager = sessionManagerStub;
                sessionManagerStub._isSessionActive.returns(true);
                _this.analytics.trackAction(action);
                var screenInfo = _this.analytics._autoCollector._getScreenResolution();
                QUnit.assert.equal(event.name, 'action');
                QUnit.assert.equal(event.iKey, 'xyz');
                QUnit.assert.equal(event.time, date2.toISOString());
                QUnit.assert.equal(event.data['s']['v']['n'], 'page1');
                QUnit.assert.equal(event.data['s']['v']['v'], 'title');
                QUnit.assert.equal(event.data['s']['v']['u'], 'uri');
                QUnit.assert.equal(event.data['s']['v']['c'], 'class');
                QUnit.assert.equal(event.data['s']['v']['h'], screenInfo.h);
                QUnit.assert.equal(event.data['s']['v']['w'], screenInfo.w);
                QUnit.assert.equal(event.data['s']['v']['g'], screenInfo.vH);
                QUnit.assert.equal(event.data['s']['v']['d'], screenInfo.vW);
                QUnit.assert.equal(event.data['s']['v']['o'], screenInfo.cd);
                QUnit.assert.equal(event.data['s']['v']['s'], screenInfo.sR);
                QUnit.assert.equal(event.data['s']['r']['u'], 'refUri');
                QUnit.assert.equal(event.data['s']['r']['p'], 'refPage');
                QUnit.assert.equal(event.data['prop'], 'val');
                QUnit.assert.equal(event.data['s']['t']['n'], 'cart');
                QUnit.assert.equal(event.data['s']['t']['t'], 'LClick');
                QUnit.assert.equal(event.data['s']['t']['m'], 'elId');
                QUnit.assert.equal(event.data['s']['t']['c'], 'class');
                QUnit.assert.equal(event.data['s']['t']['g'], 'uri2');
                QUnit.assert.equal(event.data['s']['t']['o'], true);
                QUnit.assert.equal(event.data['s']['t']['e'], 10);
                QUnit.assert.equal(event.data['s']['k']['c'], '20X12');
            }
        });
        this.testCase({
            name: 'Verify localized texts are decoded',
            test: function () {
                var appInsightsCoreStub = sinon.createStubInstance(AppInsightsCore);
                _this.analytics._applicationInsightsCore = appInsightsCoreStub;
                var view = {
                    name: '名字',
                    uri: '页面',
                    referrer: {
                        uri: '前一个页面'
                    }
                };
                var action = {
                    name: 'cart',
                    type: ActionType.leftClick,
                    viewTarget: '点击页面'
                };
                var event;
                appInsightsCoreStub.track.callsFake(function (arg) {
                    event = arg;
                });
                _this.analytics.trackView(view);
                _this.analytics.trackAction(action);
                QUnit.assert.equal(event.data['s']['v']['n'], '名字');
                QUnit.assert.equal(event.data['s']['v']['u'], '页面');
                QUnit.assert.equal(event.data['s']['r']['u'], '前一个页面');
                QUnit.assert.equal(event.data['s']['t']['g'], '点击页面');
            }
        });
        this.testCase({
            name: 'trackAction always sends view event if session is not started',
            test: function () {
                var appInsightsCoreStub = sinon.createStubInstance(AppInsightsCore);
                _this.analytics._applicationInsightsCore = appInsightsCoreStub;
                var date = new Date();
                var action = {
                    name: 'cart',
                    type: ActionType.leftClick,
                    elementId: 'elId',
                    "class": 'class',
                    clickCoordinates: '20X12',
                    viewTarget: 'uri2',
                    timestamp: date,
                    properties: {
                        'prop': 'val'
                    }
                };
                var events = [];
                appInsightsCoreStub.track.callsFake(function (arg) {
                    events.push(arg);
                });
                // Log a view to log a view in this session.
                _this.analytics.trackView({ uri: 'uri' });
                events = [];
                // Clear session key
                window.localStorage.clear();
                // Simulates that the session is over while the user is still on the page but doing nothing then finally clicks something
                _this.analytics.trackAction(action);
                QUnit.assert.equal(events.length, 2);
                QUnit.assert.equal(events[0].name, 'view');
                QUnit.assert.equal(events[1].name, 'action');
            }
        });
        this.testCase({
            name: 'Verify if local storage not availble storage is disabled',
            test: function () {
                var oldMethod = StorageManager._getLocalStorage;
                StorageManager._getLocalStorage = function () { return null; };
                _this.analytics = new Analytics();
                _this.analytics.initialize({ ingestionKey: 'xyz' });
                QUnit.assert.ok(_this.analytics._storageDisabled);
                StorageManager._getLocalStorage = oldMethod;
            }
        });
    };
    return AnalyticsTest;
}(TestClass));
export { AnalyticsTest };
//# sourceMappingURL=AnalyticsTest.js.map