import { __extends } from "tslib";
import { TestClass } from '../TestClass';
import * as sinon from 'sinon';
import SessionManager from '../../src/properties/SessionManager';
import Properties from '../../src/properties/Properties';
import { Utils, CoreUtils } from '@microsoft/1ds-core-js';
var PropertiesTest = /** @class */ (function (_super) {
    __extends(PropertiesTest, _super);
    function PropertiesTest() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PropertiesTest.prototype.testInitialize = function () {
        this.sessionManagerStub = sinon.createStubInstance(SessionManager);
        this.properties = new Properties();
    };
    PropertiesTest.prototype.testCleanup = function () {
        Utils.deleteCookie('_msei');
    };
    PropertiesTest.prototype.registerTests = function () {
        var _this = this;
        this.testCase({
            name: 'Setup will set the _msei cookie',
            test: function () {
                Utils.deleteCookie('_msei');
                _this.properties.setup(_this.sessionManagerStub);
                QUnit.assert.ok(Utils.getCookie('_msei'));
            }
        });
        this.testCase({
            name: 'Signal will get common properties added',
            test: function () {
                _this.properties.setup(_this.sessionManagerStub);
                var event = {
                    name: 'test',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            e: {}
                        }
                    }
                };
                _this.properties.processTelemetry(event);
                QUnit.assert.ok(event.data['s']['u']['d']);
                QUnit.assert.ok(event.data['s']['z']['v']);
                QUnit.assert.ok(event.data['s']['h']['d']);
                QUnit.assert.ok(event.data['s']['e']['z']);
                QUnit.assert.ok(event.data['s']['e']['s'], 'Web');
            }
        });
        this.testCase({
            name: 'Signal will be passed to session manager to process',
            test: function () {
                _this.properties.setup(_this.sessionManagerStub);
                var event = {
                    name: 'test',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            e: {}
                        }
                    }
                };
                _this.properties.processTelemetry(event);
                QUnit.assert.ok(_this.sessionManagerStub._processEvent.calledOnce);
            }
        });
        this.testCase({
            name: 'Signal can be overridden with user fields',
            test: function () {
                _this.properties.setup(_this.sessionManagerStub);
                _this.properties.setUser({
                    email: 'clippy@msoffice.com',
                    name: 'clippy',
                    localId: 'id',
                    authId: 'orig_clippy',
                    authType: 'MSA'
                });
                var event = {
                    name: 'test',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            e: {}
                        }
                    }
                };
                _this.properties.processTelemetry(event);
                QUnit.assert.equal(event.data['s']['u']['d'], 'id');
                QUnit.assert.equal(event.data['s']['u']['a'], 'orig_clippy');
                QUnit.assert.equal(event.data['s']['u']['u'], 'MSA');
                QUnit.assert.equal(event.data['s']['u']['e'], 'clippy@msoffice.com');
                QUnit.assert.equal(event.data['s']['u']['n'], 'clippy');
            }
        });
        this.testCase({
            name: 'Signal can contain custom properties',
            test: function () {
                _this.properties.setup(_this.sessionManagerStub);
                _this.properties.setProperty('test', 0);
                var event = {
                    name: 'test',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            e: {}
                        }
                    }
                };
                _this.properties.processTelemetry(event);
                QUnit.assert.equal(event.data['test'], 0);
            }
        });
        this.testCase({
            name: 'Processing the signal removes empty properties',
            test: function () {
                _this.properties.setup(_this.sessionManagerStub);
                var event = {
                    name: 'test',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            e: {}
                        },
                        test: ''
                    }
                };
                _this.properties.processTelemetry(event);
                QUnit.assert.notOk(event.data['test']);
            }
        });
    };
    return PropertiesTest;
}(TestClass));
export { PropertiesTest };
//# sourceMappingURL=PropertiesTest.js.map