import { __extends } from "tslib";
import { TestClass } from '../TestClass';
import StorageManager from '../../src/properties/StorageManager';
import SessionManager from '../../src/properties/SessionManager';
import * as sinon from 'sinon';
import { CoreUtils } from '@microsoft/1ds-core-js';
var extendTime = 1800000; //30 mins
var SessionManagerTest = /** @class */ (function (_super) {
    __extends(SessionManagerTest, _super);
    function SessionManagerTest() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SessionManagerTest.prototype.testInitialize = function () {
        this.storageManagerStub = sinon.createStubInstance(StorageManager);
        this.sessionManager = new SessionManager(this.storageManagerStub);
        this.time = new Date().getTime();
    };
    SessionManagerTest.prototype.registerTests = function () {
        var _this = this;
        this.testCase({
            name: 'Validate session active',
            test: function () {
                // When no session is there
                QUnit.assert.notOk(_this.sessionManager._isSessionActive(_this.time));
                // Add session information
                _this.storageManagerStub._getProperty.returns('id;' + _this.time + ';' + (_this.time + extendTime));
                QUnit.assert.ok(_this.sessionManager._isSessionActive(_this.time, true));
                // Expired if time is more than the extended time
                QUnit.assert.notOk(_this.sessionManager._isSessionActive(_this.time + extendTime + 1, true));
            }
        });
        this.testCase({
            name: 'Custom signals do not get Session Id if no session is active',
            test: function () {
                var event = {
                    name: 'test',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            u: {}
                        }
                    }
                };
                QUnit.assert.notOk(_this.sessionManager._isSessionActive(_this.time));
                _this.sessionManager._processEvent(event, false, false);
                QUnit.assert.equal(event.data['s']['s']['i'], '');
            }
        });
        this.testCase({
            name: 'Hit signals start a session',
            test: function () {
                QUnit.assert.notOk(_this.sessionManager._isSessionActive(_this.time));
                var view = {
                    name: 'view',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            u: {}
                        }
                    }
                };
                _this.sessionManager._processEvent(view, false, true);
                QUnit.assert.ok(_this.sessionManager._isSessionActive(Date.parse(view.time)));
                QUnit.assert.ok(view.data['s']['s']['i']);
                QUnit.assert.equal(view.data['s']['s']['d'], 0);
                _this.sessionManager = new SessionManager(_this.storageManagerStub);
                var action = {
                    name: 'action',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            u: {}
                        }
                    }
                };
                _this.sessionManager._processEvent(action, false, true);
                QUnit.assert.ok(action.data['s']['s']['i']);
                QUnit.assert.equal(action.data['s']['s']['d'], 0);
            }
        });
        this.testCase({
            name: 'Hit signals extend a session',
            test: function () {
                var sessionStored;
                var oldSessionExtendedDuration = _this.time + extendTime;
                _this.storageManagerStub._getProperty.returns('id;' + _this.time + ';' + oldSessionExtendedDuration);
                _this.storageManagerStub._setProperty.callsFake(function (arg1, arg2) {
                    sessionStored = arg2;
                });
                _this.sessionManager = new SessionManager(_this.storageManagerStub);
                var view = {
                    name: 'view',
                    time: CoreUtils.toISOString(new Date(_this.time + 10)),
                    data: {
                        s: {
                            u: {}
                        }
                    }
                };
                _this.sessionManager._processEvent(view, false, true);
                QUnit.assert.ok(_this.sessionManager._isSessionActive(Date.parse(view.time)));
                QUnit.assert.equal(view.data['s']['s']['i'], 'id');
                QUnit.assert.equal(view.data['s']['s']['d'], 10);
                QUnit.assert.equal(parseInt(sessionStored.split(';')[2], 10) - oldSessionExtendedDuration, 10);
            }
        });
        this.testCase({
            name: 'All signals contain session id if there is an active session',
            test: function () {
                _this.storageManagerStub._getProperty.returns('id;' + _this.time + ';' + (_this.time + extendTime));
                _this.sessionManager = new SessionManager(_this.storageManagerStub);
                var event = {
                    name: 'test',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            u: {}
                        }
                    }
                };
                _this.sessionManager._processEvent(event, false, false);
                QUnit.assert.equal(event.data['s']['s']['i'], 'id');
                QUnit.assert.notOk(event.data['s']['s']['d']);
            }
        });
        this.testCase({
            name: 'TimeSinceVisit is set correctly',
            test: function () {
                var view = {
                    name: 'view',
                    time: CoreUtils.toISOString(new Date(_this.time)),
                    data: {
                        s: {
                            u: {}
                        }
                    }
                };
                // New user does not have tims since visit set.
                _this.sessionManager._processEvent(view, true, true);
                QUnit.assert.notOk(view.data['s']['u']['t']);
                // Second view in the session does not have the TimeSinceVisit
                _this.sessionManager._processEvent(view, false, true);
                QUnit.assert.notOk(view.data['s']['u']['t']);
                // Set up a previous expired session that ended 5ms ago
                _this.sessionManager = new SessionManager(_this.storageManagerStub);
                _this.storageManagerStub._getProperty.returns('id;' + (_this.time - 10) + ';' + (_this.time - 5));
                _this.sessionManager._processEvent(view, false, true);
                QUnit.assert.equal(view.data['s']['u']['t'], 5);
            }
        });
        this.testCase({
            name: 'New or Returning User is set correctly',
            test: function () {
                var view = {
                    name: 'view',
                    time: CoreUtils.toISOString(new Date()),
                    data: {
                        s: {
                            u: {}
                        }
                    }
                };
                var sessionStored;
                _this.storageManagerStub._setProperty.callsFake(function (arg1, arg2) {
                    sessionStored = arg2;
                });
                // Process telemetry will add new user if no cookie is set. 
                _this.sessionManager._processEvent(view, true, true);
                QUnit.assert.equal(view.data['s']['u']['s'], 'new');
                _this.storageManagerStub._getProperty.returns(sessionStored);
                // Second view in the session still marks the user as new for entire session but user cookie is not created.
                _this.sessionManager._processEvent(view, false, true);
                QUnit.assert.equal(view.data['s']['u']['s'], 'new');
                // Mock new session but user cookie is not created.
                _this.storageManagerStub._getProperty.returns(null);
                _this.sessionManager._processEvent(view, false, true);
                QUnit.assert.equal(view.data['s']['u']['s'], 'returning');
            }
        });
    };
    return SessionManagerTest;
}(TestClass));
export { SessionManagerTest };
//# sourceMappingURL=SessionManagerTest.js.map