/*
* Analytics.ts
* @author Abhilash Panwar (abpanwar)
* @copyright Microsoft 2020
* File containing Analytics object.
*/
import { AppInsightsCore, CoreUtils, IExtendedTelemetryItem, Utils, isDocumentObjectAvailable, getDocument, IExtendedConfiguration } from '@microsoft/1ds-core-js';
import { PostChannel, RT_PROFILE, IChannelConfiguration } from '@microsoft/1ds-post-js';
import { IEvent, IView, IAnalyticsConfiguration, IAction, IUser } from './analytics/DataModels';
import { IScreenResolution } from './analytics/InternalDataModels';
import {
  _Extensions, _SignalExtKeys, _PageViewType, _SharedKeys, _ViewExtKeys, _ReferrerExtKeys, _EIPreviousKey,
  _ViewEventName, _ViewSchemaVersion, _ActionEventName, _ActionSchemaVersion, _ActionExtKeys, _ClickExtKeys, _emptyString, _UserMappingEventName, _UserMappingEventVersion, _UserMappingStorageKey, _DayToMilliseconds
} from './properties/Constants';
import Properties from './properties/Properties';
import Autocollector from './analytics/Autocollector';
import StorageManager from './properties/StorageManager';
import SessionManager from './properties/SessionManager';
import { _getPathNameFromUri, _hashCode } from './analytics/AnalyticsHelper';

const d = decodeURI; // for code minimization purpose
/**
 * Engagement Insights class that is used to send events.
 */
export default class Analytics {
  private _applicationInsightsCore: AppInsightsCore;
  private _postChannel: PostChannel;
  private _properties: Properties = new Properties();
  private _configuration: IAnalyticsConfiguration;
  private _isInitialized = false;
  private _lastView: IView;
  private _lastViewTime: Date;
  private _previousView: string;
  private _storageDisabled: boolean;
  private _storageManager: StorageManager;
  private _sessionManager: SessionManager;
  private _autoCollector = new Autocollector();  
  private _invalidSignalPrefix = _Extensions._SignalExt + '.';
  private _invalidSignalLongPrefix = _Extensions._SignalLongExt + '.'
  private _user: IUser;
  private _setUserBeforeInitialize: boolean;

  /**
   * Initializes the Analytics class according to the passed configuration. It will send autocollected events if the passed
   * configuration specifies it.
   * @param configuration the configuration used to initialize the Analytics class
   */
  initialize(configuration: IAnalyticsConfiguration) {
    if (!this._isInitialized) {
      this._isInitialized = true;
      this._configuration = configuration;
      this._applicationInsightsCore = new AppInsightsCore();
      this._postChannel = new PostChannel();
      let profile = {};
      profile[RT_PROFILE] = [2, 0];
      this._postChannel._loadTransmitProfiles(profile);
      this._storageDisabled = Utils.isValueAssigned(configuration.userConsent) ? !configuration.userConsent : false;
      this._storageDisabled = StorageManager._getLocalStorage() ? this._storageDisabled : true;
      this._storageManager = new StorageManager(this._storageDisabled);

      //Only enable session if the user has given consent.
      if (!this._storageDisabled) {
        this._sessionManager = new SessionManager(this._storageManager);
        this._properties.setup(this._sessionManager);
      }

      //Initialize the core
      let coreConfig: IExtendedConfiguration = {
        instrumentationKey: configuration.ingestionKey,
        endpointUrl: configuration.endpointUrl,
        disableCookiesUsage: this._storageDisabled,
        extensions: [this._properties, this._postChannel],
        extensionConfig: []
      };
      let postConfig: IChannelConfiguration = {
        ignoreMc1Ms0CookieProcessing: true
      };
      coreConfig.extensionConfig[this._postChannel.identifier] = postConfig;
      this._applicationInsightsCore.initialize(coreConfig, []);

      //Pass autocapture config to Autocollector
      this._autoCollector._autocollect(this, configuration.autoCapture);

      // send stored usermapping if setUser is called before initialize
      if (this._setUserBeforeInitialize && this._configuration.userMapping) {
        this._updateUserMapping(this._user);
      }
    }
  }

  /**
   * Tracks a custom event to be sent to Engagement Insights.
   * @param event event to be sent
   */
  trackEvent(event: IEvent) {
    event.properties = event.properties || {};
    this._sanitizeProperties(event.properties);
    this._trackEvent(event);
  }

  /**
   * Tracks a view event to be sent to Engagement Insights.
   * @param view view event to be sent
   */
  trackView(view: IView) {
    if (!this._storageDisabled) {
      if (!view || !Utils.isObject(view) || !Utils.isString(view.uri) || view.uri.length === 0) {
        throw 'View event should have uri';
      }
      this._lastViewTime = view.timestamp || new Date();
      let event: IEvent = {
        name: _ViewEventName,
        version: _ViewSchemaVersion,
        timestamp: this._lastViewTime
      };   
      let props = view.properties || {};
      this._sanitizeProperties(props);
      // There should only be a previous view loaded if the session is ongoing.
      if (this._sessionManager._isSessionActive(this._lastViewTime.getTime()), true) {
        this._previousView = this._storageManager._getProperty(_EIPreviousKey);
      }
      this._addAutocollectableFieldsToView(view);
      this._addViewInfo(view, props);
      this._addScreenInfo(props);
      this._lastView = view;
      event.properties = props;
      this._trackEvent(event);
      this._storageManager._setProperty(_EIPreviousKey, view.uri);
    }
  }

  /**
   * Tracks an action event to be sent to Engagement Insights.
   * @param action action event to be sent
   * @param properties custom properties that are part of this event as defined by the emitter.
   */
  trackAction(action: IAction, properties?: { [key: string]: string | number | boolean }) {
    if (!this._storageDisabled) {
      if (!this._lastView) {
        throw 'Views should be instrumented before actions are sent';
      }
      if (!Utils.isValueAssigned(action.type)) {
        throw 'Action event should have type';
      }
      let actionTime = action.timestamp || new Date();
      // If this is the action event that starts a session then send a view event from the last view before the action is sent. 
      if (!this._sessionManager._isSessionActive(actionTime.getTime())) {
        this._lastView.timestamp = actionTime;
        this.trackView(this._lastView);
      }   
      let event: IEvent = {
        name: _ActionEventName,
        version: _ActionSchemaVersion,
        timestamp: actionTime
      };    
      let props = action.properties || {};
      this._sanitizeProperties(props);
      this._addViewInfo(this._lastView, props);
      this._addScreenInfo(props);
      //Add action properties
      props[_Extensions._SignalExt][_SignalExtKeys._Action] = {};
      props[_Extensions._SignalExt][_SignalExtKeys._Action][_SharedKeys._NameKey] = action.name;
      props[_Extensions._SignalExt][_SignalExtKeys._Action][_SharedKeys._Type] = action.type;
      props[_Extensions._SignalExt][_SignalExtKeys._Action][_ActionExtKeys._ElementId] = action.elementId;
      props[_Extensions._SignalExt][_SignalExtKeys._Action][_SharedKeys._Class] = action.class;
      props[_Extensions._SignalExt][_SignalExtKeys._Action][_ActionExtKeys._Target] = d(action.viewTarget);
      //Elapsed time
      props[_Extensions._SignalExt][_SignalExtKeys._Action][_ActionExtKeys._ElapsedTime] = actionTime.getTime() - this._lastViewTime.getTime();
      //Outbound. Only check if there is a view target and we have an origin for the webpage.
      if (action.viewTarget && this._properties.origin) {
        props[_Extensions._SignalExt][_SignalExtKeys._Action][_ActionExtKeys._IsOutbound] = !action.viewTarget.startsWith(this._properties.origin);
      }
      props[_Extensions._SignalExt][_SignalExtKeys._Click] = {};
      props[_Extensions._SignalExt][_SignalExtKeys._Click][_ClickExtKeys._Coordinates] = action.clickCoordinates;
      event.properties = props;
      this._trackEvent(event);
    }
  }

  /**
   * Teardown the SDK, and flushes all events currenlty in the queue.
   * After this method the SDK can be re-initialized with a new configuration.
   */
  teardown() {
    if (this._isInitialized) {
      this._postChannel.teardown();
      this._isInitialized = false;
      //Reset properties      
      this._properties = new Properties();
      this._autoCollector._removeAllListeners();
    }
  }

  /**
   * Set custom property to be sent with every event.
   * @param name name of property
   * @param value value for the property
   */
  setProperty(name: string, value: string | number | boolean | Date) {
    if (this._isValidKey(name)) {
      this._properties.setProperty(name, value);
    }
  }

  /**
   * Set user details to be sent with every event. Update UserMapping depending on whether the SDK is initialized or not
   * @param user user details
   */
  setUser(user: IUser) {
    this._properties.setUser(user);
    if (this._isInitialized && this._configuration.userMapping) {
      this._updateUserMapping(user);
    } else {
      this._user = user;
      this._setUserBeforeInitialize = true;
    }
  }

  private _trackEvent(event: IEvent) {
    if (this._isInitialized) {
      let item: IExtendedTelemetryItem = {
        name: event.name,
        iKey: event.ingestionKey || this._configuration.ingestionKey
      };
      let props: { [key: string]: string | number | boolean | string[] | number[] | boolean[] | object } = event.properties;
      let time: Date = event.timestamp ? event.timestamp : new Date();
      item.time = CoreUtils.toISOString(time);
      props[_Extensions._SignalExt][_SignalExtKeys._ExtKey] = {};
      props[_Extensions._SignalExt][_SignalExtKeys._ExtKey][_SharedKeys._Id] = Utils.createGuid();
      if (!CoreUtils.isNullOrUndefined(event.version)) {
        props[_Extensions._SignalExt][_SignalExtKeys._ExtKey][_SharedKeys._Version] = event.version;
      }
      item.data = props;
      this._applicationInsightsCore.track(item);
    }
  }

  private _sanitizeProperties(props: { [key: string]: string | number | boolean | string[] | number[] | boolean[] | object }) {
    // Don't allow customer to send any fields in the event extension. It is reserved for EI.
    Utils.arrForEach(Utils.objKeys(props), (key) => {
      if (!this._isValidKey(key)) {
        delete props[key];
      }
    });
    props[_Extensions._SignalExt] = {};
  }

  private _isValidKey(key: string): boolean {
    key = key.toLowerCase();
    return !(key.length < 3 || key.startsWith(this._invalidSignalPrefix) || key.startsWith(this._invalidSignalLongPrefix) || key === _Extensions._SignalLongExt);
  }

  private _addScreenInfo(props: { [key: string]: string | number | boolean | string[] | number[] | boolean[] | object }) {
    // Get screen resolution
    let screen: IScreenResolution = this._autoCollector._getScreenResolution();
    if (screen) {
      props[_Extensions._SignalExt][_SignalExtKeys._View][_ViewExtKeys._Height] = screen.h;
      props[_Extensions._SignalExt][_SignalExtKeys._View][_ViewExtKeys._Width] = screen.w;
      props[_Extensions._SignalExt][_SignalExtKeys._View][_ViewExtKeys._VpHeight] = screen.vH;
      props[_Extensions._SignalExt][_SignalExtKeys._View][_ViewExtKeys._VpWidth] = screen.vW;
      props[_Extensions._SignalExt][_SignalExtKeys._View][_ViewExtKeys._ColorDepth] = screen.cd;
      props[_Extensions._SignalExt][_SignalExtKeys._View][_ViewExtKeys._SResolution] = screen.sR;
    }
  }

  private _addViewInfo(view: IView, props: { [key: string]: string | number | boolean | string[] | number[] | boolean[] | object }) {
    props[_Extensions._SignalExt][_SignalExtKeys._View] = {};
    props[_Extensions._SignalExt][_SignalExtKeys._View][_SharedKeys._Title] = view.title;
    props[_Extensions._SignalExt][_SignalExtKeys._View][_SharedKeys._Type] = _PageViewType;
    props[_Extensions._SignalExt][_SignalExtKeys._View][_SharedKeys._Uri] = d(view.uri);    
    props[_Extensions._SignalExt][_SignalExtKeys._View][_SharedKeys._NameKey] = d(view.name);
    props[_Extensions._SignalExt][_SignalExtKeys._View][_SharedKeys._Class] = view.viewClass;       
    props[_Extensions._SignalExt][_SignalExtKeys._View][_ViewExtKeys._PreviousViews] = this._previousView;    
       
    if (view.referrer && Utils.isObject(view.referrer)) {
      props[_Extensions._SignalExt][_SignalExtKeys._Referrer] = {};
      props[_Extensions._SignalExt][_SignalExtKeys._Referrer][_ReferrerExtKeys._PageTitle] = view.referrer.pageTitle;
      props[_Extensions._SignalExt][_SignalExtKeys._Referrer][_SharedKeys._Uri] = d(view.referrer.uri);
    }
  }

  private _addAutocollectableFieldsToView(view: IView) {
    view.name = view.name || this._autoCollector._getPageNameFromPath(_getPathNameFromUri(view.uri));    
    if ( isDocumentObjectAvailable) {
      view.referrer = view.referrer || this._autoCollector._getReferrer(getDocument());
    }
  }

  private _updateUserMapping(user: IUser) {
    let time = CoreUtils.dateNow();
    let localId = user.localId || this._properties._msei;
    if (localId && user.authId) {
      let usermappingKey = _hashCode(localId + ',' + user.authId).toString();
      let storedMapping = JSON.parse(this._storageManager._getProperty(_UserMappingStorageKey)) || {};
      if (!CoreUtils.hasOwnProperty(storedMapping, usermappingKey) || time - parseInt(storedMapping[usermappingKey]) > 7 * _DayToMilliseconds) {
        storedMapping[usermappingKey] = time.toString();
        this._storageManager._setProperty(_UserMappingStorageKey, JSON.stringify(storedMapping));
        let event: IEvent = {
          name: _UserMappingEventName,
          version: _UserMappingEventVersion,
          properties: {}
        };
        this._sanitizeProperties(event.properties);
        this._trackEvent(event);  
      }

      if (Utils.objKeys(storedMapping).length > 5) {
        this._deleteOldestUserMapping(storedMapping);
      }
    }
  }

  private _deleteOldestUserMapping(storedMapping: { [key: string]: string}) {
    let oldestMappingKey = '';
    let oldestTimestamp = Number.MAX_VALUE;
    Utils.arrForEach(Utils.objKeys(storedMapping), (key)=>{
      if (parseInt(storedMapping[key]) < oldestTimestamp) {
        oldestMappingKey = key;
        oldestTimestamp = parseInt(storedMapping[key]);
      }
    });
    delete storedMapping[oldestMappingKey];
    this._storageManager._setProperty(_UserMappingStorageKey, JSON.stringify(storedMapping));
  }
  
}
