/**
* Autocollector.ts
* @author Abhilash Panwar (abpanwar) Hector Hernandez (hectorh)
* @copyright Microsoft 2020
* File containing class for autocollecting signals for EI.
*/
import { Utils, getLocation, getDocument, isDocumentObjectAvailable, getWindow, isWindowObjectAvailable } from '@microsoft/1ds-core-js';
import Analytics from '../Analytics';
import { IView, IAutoCapture, IAction } from './DataModels';
import { IScreenResolution } from './InternalDataModels';
import { _isRightClick, _isLeftClick, _isMiddleClick, _isKeyboardEnter, _isKeyboardSpace, _clickCaptureInputTypes, _getClickTarget, _getContent, } from './AnalyticsHelper';
import { ActionType } from './Enums';
import { _emptyString } from '../properties/Constants';
import { IReferrer } from '../Index';

const clickCaptureElements = { A: true, BUTTON: true, AREA: true, INPUT: true };
/**
 * Class for autocollecting signals.
 */
export default class Autocollector {
  private _analytics: Analytics;
  /**
   * Autocollects signals based on the config passed.
   * @param analytics The analytics class to use to send signals.
   * @param config The config detailing what needs to be auto collected.   
   */
  _autocollect(analytics: Analytics, config: IAutoCapture) {    
    this._analytics = analytics;
    let captureView = this._captureView.bind(this);
    if (config && Utils.isObject(config)) {
      if (config.view) {
        captureView();
        let win = getWindow();
        const _replaceState = win.history.replaceState;
        const _pushState = win.history.pushState;
        win.history.replaceState = function () {
          _replaceState.apply(win.history, arguments);
          captureView();
        };
        win.history.pushState = function () {
          _pushState.apply(win.history, arguments);
          captureView();
        };
      }
      if (config.click) {
        this._captureClicks();
      }
    }
  }

  /**
   * Gets the screen resolution.
   */
  _getScreenResolution(): IScreenResolution {
    if (isDocumentObjectAvailable && isWindowObjectAvailable) {
      let win = getWindow();
      let doc = getDocument();
      return {
        h: screen.height,
        w: screen.width,
        cd: screen.colorDepth,
        vW: Math.max(doc.documentElement.clientWidth, win.innerWidth || 0),
        vH: Math.max(doc.documentElement.clientHeight, win.innerHeight || 0),
        sR: screen.height && screen.width ? screen.height + 'X' + screen.width : _emptyString
      };
    }
    return null;
  }

  _getPageNameFromPath(pageName: string): string {
    pageName = pageName.replace(/\/$/, "");
    let fragments = pageName.split('/');
    if (fragments && fragments[fragments.length - 1] !== _emptyString) {
      pageName = fragments[fragments.length - 1];
    } else {
      pageName = '/';
    }
    return pageName;
  }

  _getReferrer(doc: Document): IReferrer {
    return {
      uri: doc.referrer
    };
  }

  _removeAllListeners() {
    let win = getWindow();
    let doc = getDocument();
    if (win && win.addEventListener) {
        // IE9 onwards addEventListener is available, 'click' event captures mouse click. mousedown works on other browsers
        var event = (navigator.appVersion.indexOf('MSIE') !== -1) ? 'click' : 'mousedown';
        win.removeEventListener(event, (evt) => { this._processClick(evt); }, false);
        win.removeEventListener('keyup', (evt) => { this._processClick(evt); }, false);
    }
  }

  private _captureView() {
    let loc = getLocation();
    if (isDocumentObjectAvailable && loc) {
      let doc = getDocument();     
      let view: IView = {
        uri: loc.href,
        title: doc.title.substring(0, 150),
        name: this._getPageNameFromPath(loc.pathname || _emptyString),
        referrer: this._getReferrer(doc)
      };
      this._analytics.trackView(view);
    }
  }

  private _captureClicks() {
    let win = getWindow();
    let doc = getDocument();
    if (win && win.addEventListener) {
        // IE9 onwards addEventListener is available, 'click' event captures mouse click. mousedown works on other browsers
        var event = (navigator.appVersion.indexOf('MSIE') !== -1) ? 'click' : 'mousedown';
        win.addEventListener(event, (evt) => { this._processClick(evt); }, false);
        win.addEventListener('keyup', (evt) => { this._processClick(evt); }, false);
    }
  }

  private _processClick(clickEvent: any) {
    let element = clickEvent.srcElement || clickEvent.target; 
    let sendAction = false;  
    // Check if we should capture the element
    while (element && element.tagName) {
      if (element.control && clickCaptureElements[element.control.tagName.toUpperCase()]) {
        element = element.control;
      }
      if (!clickCaptureElements[element.tagName.toUpperCase()]) {
        element = element.parentElement || element.parentNode;
        continue;
      } else {
        // Check allowed INPUT types
        sendAction = element.tagName.toUpperCase() === 'INPUT' ? _clickCaptureInputTypes[element.type.toUpperCase()] : true;
        break;
      }
    }
    if (sendAction) {
      // Capture the action type
      let actionType: string;
      if (_isRightClick(clickEvent as MouseEvent)) {
        actionType = ActionType.rightClick;
      } else if (_isLeftClick(clickEvent as MouseEvent)) {
        actionType = ActionType.leftClick;
      } else if (_isKeyboardEnter(clickEvent as KeyboardEvent)) {
        actionType = ActionType.keyboardEnter;
      } else if (_isKeyboardSpace(clickEvent as KeyboardEvent)) {
        actionType = ActionType.keyboardSpace;
      } else if (_isMiddleClick(clickEvent as MouseEvent)) {
        actionType = ActionType.middleClick;
      } else {
        return;
      }    
      const content = _getContent(element);
      const action: IAction = {
        name: content.n,
        type: actionType,
        clickCoordinates: clickEvent.pageX + 'X' + clickEvent.pageY,
        class: content.c,
        elementId: content.id,
        viewTarget: _getClickTarget(element)
      };
      this._analytics.trackAction(action);
    }
  }
}
