/*
* Properties.ts
* @author Abhilash Panwar (abpanwar) Hector Hernandez (hectorh)
* @copyright Microsoft 2020
* File containing Properties plugin to collect common properties.
*/
import { BaseTelemetryPlugin, IProcessTelemetryContext, getLocation, Utils, IExtendedTelemetryItem, IEventProperty, CoreUtils } from '@microsoft/1ds-core-js';
import { _Extensions, _SignalExtKeys, _SharedKeys, _SDKVersion, _emptyString, _UserExtKeys, _MSEI, _ViewEventName, _ActionEventName } from './Constants';
import SessionManager from './SessionManager';
import { IUser } from '../analytics/DataModels';

const mseiCookieExpiryDays = 365;

/**
 * This class adds a telemetry plugin to populate autocollected properties. 
 */
export default class Properties extends BaseTelemetryPlugin {
  private _timeZone: string;
  private _domain: string;
  private _user: IUser = {};
  private _mseiCreated: boolean;
  private _props:  { [key: string]: string | number | boolean | Date } = {};
  private _sessionManager: SessionManager;
  public origin: string; 
  public _msei: string;

  constructor() {
    super();
    //Get time zone
    let timeZone = new Date().getTimezoneOffset();
    let minutes = timeZone % 60;
    let hours = (timeZone - minutes) / 60;
    let timeZonePrefix = '+';
    if (hours > 0) {
      timeZonePrefix = '-';
    }
    hours = Math.abs(hours);
    minutes = Math.abs(minutes);
    this._timeZone = timeZonePrefix + (hours < 10 ? '0' + hours : hours.toString()) + ':' + (minutes < 10 ? '0' + minutes : minutes.toString());

    //Get domain
    let windowLocation = getLocation();
    if (windowLocation) {
      let domain = windowLocation.hostname;
      this.origin = windowLocation.origin;
      if (domain) {
        this._domain = windowLocation.protocol === 'file:' ? 'local' : domain;
      }
    }
  }

  setup(SessionManager: SessionManager) {
    this._msei = Utils.getCookie(_MSEI);
    if (!this._msei) {
      this._msei = Utils.createGuid();
      Utils.setCookie(_MSEI, this._msei, mseiCookieExpiryDays);
      this._mseiCreated = true;
    }
    this._sessionManager = SessionManager;
  }

  /**
   * Process the current telemetry item.
   * @param event event to process
   * @param itemCtx context information for the core telemetry library
   */
  processTelemetry(event: IExtendedTelemetryItem, itemCtx?: IProcessTelemetryContext): void {
    event.data[_Extensions._SignalExt][_SignalExtKeys._ExtKey][_SharedKeys._TimeZone] = this._timeZone;
    event.data[_Extensions._SignalExt][_SignalExtKeys._ExtKey][_SharedKeys._Source] = 'Web';
    event.data[_Extensions._SignalExt][_SignalExtKeys._Sdk] = {};
    event.data[_Extensions._SignalExt][_SignalExtKeys._Sdk][_SharedKeys._Version] = _SDKVersion;
    event.data[_Extensions._SignalExt][_SignalExtKeys._Host] = {};
    event.data[_Extensions._SignalExt][_SignalExtKeys._Host][_SharedKeys._Domain] = this._domain;
    
    //Add user fields
    event.data[_Extensions._SignalExt][_SignalExtKeys._User] = {};
    event.data[_Extensions._SignalExt][_SignalExtKeys._User][_UserExtKeys._LocalId] = this._user.localId || this._msei;
    event.data[_Extensions._SignalExt][_SignalExtKeys._User][_UserExtKeys._AuthId] = this._user.authId;
    event.data[_Extensions._SignalExt][_SignalExtKeys._User][_UserExtKeys._AuthType] = this._user.authType;
    event.data[_Extensions._SignalExt][_SignalExtKeys._User][_UserExtKeys._Email] = this._user.email;
    event.data[_Extensions._SignalExt][_SignalExtKeys._User][_SharedKeys._NameKey] = this._user.name;
    
    //Pass event to session manager to process and add required session fields.
    this._sessionManager._processEvent(event, this._mseiCreated, event.name === _ViewEventName || event.name === _ActionEventName);    
    
    //Add common custom properties
    this._addPropsIfNotAlreadyAdded(event.data);

    //Delete empty properties
    this._removeEmptyKeys(event.data);
    this.processNext(event, itemCtx);
  }

  setUser(user: IUser) {
    this._user = user;
  }

  setProperty(name: string, value: string | number | boolean | Date) { 
    //1DS SDK transport will validate name and value so not duplicating it here.     
    this._props[name] = value;    
  }

  private _addPropsIfNotAlreadyAdded(data: { [key: string]: string | number | boolean | string[] | number[] | boolean[] | IEventProperty | object}) {
    Utils.arrForEach(Utils.objKeys(this._props), (key) => {
      if (!Utils.isValueAssigned(data[key])) {
        data[key] = this._props[key];
      }
    });
  }

  private _removeEmptyKeys(input: object) {
    Utils.arrForEach(Utils.objKeys(input), (key) => {
      if (input[key] && Utils.isObject(input[key]) && !CoreUtils.isDate(input[key])) {
        if (Utils.objKeys(input[key]).length === 0) {
          delete input[key];
        } else {
          this._removeEmptyKeys(input[key]);
        }
      } else if (!Utils.isValueAssigned(input[key])) {
        delete input[key];
      }
    });
  }
}
